use strict;
use warnings;

use Win32::Sound;
use Tk;
use Tk::FileSelect;
use Tk::Scale;
use Tk::Dialog;
use Tk::LabFrame;
use IO::File;

our $font = ['Helvetica',11,'bold'];
our $b_width = 3;


our $button_changing; #temp variable to store the name of the button being changed with the Assign sub-routine
our $Configuration_window;

#Variables from setup file, if no setup file is found these will be the default settings.
our $volume_visible = 1;
our $orient_vol = 'vertical';
our $font_size = 12;
our $default_flag = 1;
our $columns;
our $rows;
our $number_of_buttons = 4;
our $volume = 75;
our @buttons;
our %file;
&Retrieve_Settings;

our $button_font = ['Helvetica',$font_size,'bold'];


our %button;
our %frame;

our $main = new MainWindow(-title=>'Sound Box');
	our $button_main_frame = $main->Frame()->pack(-side=>'bottom',-expand=>1,-fill=>'both');
	$main->iconify;
	&main_menu;
	&Create_Volume if $volume_visible;
	&Create_Buttons;
	
	
MainLoop;

################### START OF SUB-ROUNTINES ###################

sub Create_Buttons {

my $binding = 1;
	
$b_width = 0 if defined @buttons;	
foreach(@buttons){
$b_width = length($_) if (length($_) > $b_width);	 
}
$b_width += 2;

if ($default_flag){
$columns = int(sqrt($number_of_buttons));
$rows = int($number_of_buttons/$columns)+($number_of_buttons%$columns == 0?0:1);
}

foreach my $i (1..$rows){
	
	${frame{$i}} = $button_main_frame->Frame()->pack(-side=>'top',-expand=>1,-fill=>'both');
	
	foreach my $j (1..$columns){
		
		unless (($i-1)*$columns+$j > $number_of_buttons){		
			${button{$i.$j}} = ${frame{$i}}->Button(-font=>$button_font,
													-text=>(defined $buttons[(($i-1)*$columns+$j)-1]?"$buttons[(($i-1)*$columns+$j)-1]".($binding < 10?" - $binding":""):""),
													-width=>$b_width,
													-state=>(defined $buttons[(($i-1)*$columns+$j)-1]?'normal':'disabled'),
													-activebackground=>(defined $buttons[(($i-1)*$columns+$j)-1]?'#FEDA4B':'SystemButtonFace'),
													-command=>sub{Win32::Sound::Play(${file{$buttons[(($i-1)*$columns+$j)-1]}},SND_ASYNC) if defined $buttons[(($i-1)*$columns+$j)-1]}
													)->pack(-side=>'left',-pady=>2,-padx=>2,-expand=>1,-fill=>'both');
			$main->bind("<Control-KeyPress-$binding>" =>sub{Win32::Sound::Play(${file{$buttons[(($i-1)*$columns+$j)-1]}},SND_ASYNC)}) if $binding < 10; 
			$binding++;
		}#end unless
	}#end foreach my $j
}#end foreach my $i

}#end sub Create_Buttons

###########################################################################################

sub Create_Volume {
	
our $volume_frame = $main->Frame(-relief=>'sunken',-borderwidth=>2)->pack(-side=>'left',-expand=>1,-fill=>'both',-pady=>2,-padx=>2);
our $volume_label = $volume_frame->Label(-text=>"VOL",-font=>$font,-justify=>'right')->pack(-side=>'top');
our $level_frame = $volume_frame->Frame()->pack(-side=>'top');
	$level_frame->Label(-textvariable=>\$volume,-font=>$font,-justify=>'left',-width=>2)->pack(-side=>'left');
	$level_frame->Label(-text=>"%",-font=>$font)->pack(-side=>'left');
	
our $scale_frame = $volume_frame->Frame()->pack(-side=>'top',-expand=>1,-fill=>'y');
our $volume_scale = $scale_frame->Scale(-troughcolor=>'dark gray',-command=>sub{Win32::Sound::Volume("${volume}%");},-width=>12,-sliderlength=>20,-orient=>'vertical',-variable=>\$volume,-from=>100,-to=>0,-showvalue=>0)->pack(-side=>'right',-fill=>'y',-expand=>1,-padx=>5);
		$volume_scale->bind('<Leave>'=>sub{&Save_Settings});
	
our $stop_button = $volume_frame->Button(-bg=>'#E60000',-fg=>'white',-text=>'QUIET',-font=>['Helvetica',8,'bold'],
										 -activebackground=>'#FF2828',-activeforeground=>'white',-borderwidth=>4,
										 -command=>sub{Win32::Sound::Stop()})->pack(-side=>'top',-padx=>3,-pady=>3);

$main->bind('<Escape>' =>sub{Win32::Sound::Stop()}); 			

&Change_Volume;
	
sub Change_Volume {
if ($volume_visible){
	$volume_frame->pack(-side=>($orient_vol eq 'horizontal'?'top':'left'),-fill=>'both',-expand=>1);		
	$level_frame->pack(-side=>($orient_vol eq 'horizontal'?'left':'top'));
	$volume_label->pack(-side=>($orient_vol eq 'horizontal'?'left':'top'));
	$scale_frame->pack(-side=>($orient_vol eq 'horizontal'?'left':'top'),-expand=>1,-fill=>'both');
	$volume_scale->configure(-orient=>$orient_vol,-from=>($orient_vol eq 'horizontal'?0:100),-to=>($orient_vol eq 'horizontal'?100:0));
	$volume_scale->pack(-side=>($orient_vol eq 'horizontal'?'left':'right'),-fill=>($orient_vol eq 'horizontal'?'x':'y'),-expand=>1,-padx=>0);
	$stop_button->pack(-side=>($orient_vol eq 'horizontal'?'left':'top'));
	$button_main_frame->pack(-side=>($orient_vol eq 'horizontal'?'bottom':'right'),-expand=>1,-fill=>'both');
	&Save_Settings;
	}
}	

sub Volume_Visible {
	$volume_frame->destroy if !$volume_visible;
	&Create_Volume if $volume_visible;
	&Save_Settings;		
}
	
}#end sub Create_Volume

###########################################################################################

#Returns 1 if the config file is found, 0 otherwise
sub Retrieve_Settings {

my $f = 'sound_box.config';
open FP, $f or return 0;
while ( <FP> ) {
	chomp;
	next if /^\s*$/;
	
	if ($_ =~ /^(.*)(\=\>)(.*)$/){
		if ($1 ne 'Buttons' && $1 ne 'Volume' && 
		    $1 ne 'Font' && $1 ne 'Default' && 
		    $1 ne 'Rows' && $1 ne 'Columns' && 
		    $1 ne 'Volume Orientation' && $1 ne 'Volume Visible'){
			${file{$1}} = $3;
			push @buttons, $1;
		}
		if ($1 eq 'Buttons'){
			$number_of_buttons = $3;
		}
		if ($1 eq 'Volume'){
			$volume = $3;
		}
		if ($1 eq 'Font'){
			$font_size = $3;
		}
		if ($1 eq 'Default'){
			$default_flag = $3;
		}
		if ($1 eq 'Rows'){
			$rows = $3 unless $default_flag;
		}
		if ($1 eq 'Columns'){
			$columns = $3 unless $default_flag;
		}
		if ($1 eq 'Volume Orientation'){
			$orient_vol = $3;
		}
		if ($1 eq 'Volume Visible'){
			$volume_visible = $3;
		}	
	}
}
close FP;
return 1;
}#end sub Retrieve_Settings

###########################################################################################

sub Save_Settings {
	
my $config_file = new IO::File "sound_box.config", "w";

print $config_file "Buttons=>${number_of_buttons}\n";
print $config_file "Volume=>${volume}\n";
print $config_file "Font=>$font_size\n";
print $config_file "Default=>$default_flag\n";
print $config_file "Rows=>".(defined $rows?$rows:'')."\n";
print $config_file "Columns=>".(defined $columns?$columns:'')."\n";
print $config_file "Volume Orientation=>$orient_vol\n";
print $config_file "Volume Visible=>$volume_visible\n";

foreach (@buttons){
	print $config_file "${_}=>${file{$_}}\n";
}

}#end sub Save_Settings

###########################################################################################

sub Configuration_Manager {

$button_changing = $number_of_buttons;

our $file_path = (defined $buttons[0]?${file{$buttons[0]}}:'');
our @displayed_list = @buttons;
	
$Configuration_window = $main->Toplevel(-title=>'Configuration Manager',-popover=>$main);

	my $Button_numbers_frame = $Configuration_window->LabFrame(-label=>'NUMBER OF BUTTONS',-labelside => "acrosstop",-relief=>'groove',-font=>$font)->pack(-side=>'top',-expand=>1,-fill=>'both');
	my $Assign_frame = $Configuration_window->LabFrame(-label=>'ASSIGN / EDIT BUTTONS',-labelside => "acrosstop",-relief=>'groove',-font=>$font)->pack(-side=>'top');

	our $Button_list = $Assign_frame->Scrolled(qw/Listbox -setgrid 1 -exportselection 0 -height 5  -borderwidth 2 -relief sunken/,-listvariable=>\@displayed_list, -scrollbars=>'se')->pack(-side=>'top',-pady=>3,-expand=>1,-fill=>'x');
	$Assign_frame->Label(-font=>['Helvetica',10,'bold'],-text=>'PATH / FILENAME:')->pack(-side=>'top',-expand=>1,-fill=>'x',-padx=>3);
	$Assign_frame->Entry(-font=>['Helvetica',8,'bold'],-textvariable=>\$file_path,-state=>'readonly',-width=>75)->pack(-side=>'top',-expand=>1,-fill=>'x',-padx=>3);
	my $Assign_Bottom_Frame = $Assign_frame->Frame()->pack(-side=>'top',-padx=>'5');	
			our $new_button = $Assign_Bottom_Frame->Button(-text=>'NEW',-width=>7,-font=>['Helvetica',10,'bold'],-command=>sub{&New},-state=>(@buttons < $number_of_buttons?'active':'disabled'))->pack(-side=>'left',-padx=>2,-pady=>3);
			$Assign_Bottom_Frame->Button(-text=>'DELETE',-width=>7,-font=>['Helvetica',10,'bold'],-command=>sub{&Delete})->pack(-side=>'right',-padx=>2,-pady=>3);
			$Assign_Bottom_Frame->Button(-text=>'EDIT',-width=>7,-font=>['Helvetica',10,'bold'],-command=>sub{&Edit})->pack(-side=>'bottom',-padx=>2,-pady=>3);
	my $Bottom_Frame = $Configuration_window->Frame()->pack(-side=>'top');  
			$Bottom_Frame->Button(-text=>'APPLY',-width=>8,-font=>['Helvetica',13,'bold'],-command=>sub{
																						@buttons = @displayed_list;
																						&Change_num_of_buttons;
																						&Save_Settings;							
																						$Configuration_window->destroy;
																					 }
																					 )->pack(-side=>'left',-padx=>2);
			$Bottom_Frame->Button(-text=>'CANCEL',-width=>8,-font=>['Helvetica',13,'bold'],-command=>sub{$Configuration_window->destroy;})->pack(-side=>'right',-padx=>2);

			$Button_numbers_frame->Scale(-from=>1,-to=>64,-orient=>'horizontal',-font=>['Arial',11,'bold'],-showvalue=>1,-troughcolor=>'dark gray',-variable=>\$button_changing,-width=>12,-sliderlength=>20,-command=>sub{$new_button->configure(-state=>(@displayed_list < $button_changing?'active':'disabled'))})->pack(-side=>'top',-expand=>1,-fill=>'x');
			
$Button_list->bind('<ButtonRelease-1>' =>sub {$file_path =  ${file{$Button_list->get('active')}};});
$Button_list->selectionClear(0, $Button_list->index('end'));
$Button_list->activate(0);
$Button_list->selectionSet(0);    	
		
	sub Delete {
		
	    	my $index;
	    	foreach my $i (0..@displayed_list-1){
	    	$index = $i if $displayed_list[$i] eq $Button_list->get('active');	
	    	}
	
	    if (@displayed_list == 1){  
    	$displayed_list[0] = '';
		}
		
		unless (@displayed_list == 1){  
    	splice @displayed_list, $index,1;
		}
		
		$Button_list->configure(-listvariable=>\@displayed_list);  	
		$Button_list->activate($Button_list->index('end')) if $Button_list->index('active') == $Button_list->index('end');
		$Button_list->selectionSet('active');

	    foreach my $i (0..@displayed_list-1){
		$file_path = ${file{$Button_list->get('active')}} if $displayed_list[$i] eq $Button_list->get('active');	
		}
		
		$new_button->configure(-state=>(@displayed_list < $button_changing?'active':'disabled'))
				
	}#end sub Delete
	
	sub New {
		
		my $file = $main->getOpenFile(-filetypes=>[['Wave Files','.wav'],['','',],],-title=>'Choose a sound file');
		
		if (defined $file){
			my $new_name = '';
        	my @local_list = @displayed_list;
    	
			my $prompt_box = $main->Dialog(
				 -title          =>'New Button' ,
				 -buttons=>[ 'OK','Cancel'],
			);
		
			$prompt_box->Label(-text=>'Please enter a button name:',-font=>$font)->pack(-side=>'top');
			my $name_entry = $prompt_box->Entry(-font=>$font,
											-textvariable=>\$new_name,
											-validate=>'key',
											-vcmd=>sub{ 
														my $flag = 1;
														foreach (@displayed_list){$flag = 0 if $_[0] eq $_}
														if ($_[0] ne '' && $flag){
															$prompt_box->Subwidget('B_OK')->configure(-state=>'active');
														}
														else {
															$prompt_box->Subwidget('B_OK')->configure(-state=>'disabled');															
														}
													return 1;	
													}
											)->pack(-side=>'top');
											
			$prompt_box->Subwidget('B_OK')->configure(-font=>['Helvetica',11,'bold'],-state=>'disabled');
			$prompt_box->Subwidget('B_Cancel')->configure(-font=>['Helvetica',11,'bold']);
			$prompt_box->resizable(0,0);
			$prompt_box->configure(-popover=>$Configuration_window);   	
			my $button = $prompt_box->Show;
		
			if ($button eq 'OK'){
				
				push @displayed_list, $new_name;	
				${file{$new_name}} = $file;
				$new_button->configure(-state=>(@displayed_list < $button_changing?'active':'disabled'))
			}	
		}	
	}#end sub New
	
	sub Edit {
		
	my $new_name = $Button_list->get('active');
	my $old_name = $Button_list->get('active');
    my @local_list = @displayed_list;
    	
		my $prompt_box = $main->Dialog(
			 -title          =>'Edit Button' ,
			 -buttons=>['Done','Cancel','Edit File'],
		);
		
		$prompt_box->Label(-text=>"Please edit the name of the button\nand/or the file/path:",-font=>$font,-justify=>'center')->pack(-side=>'top');
		my $name_entry = $prompt_box->Entry(-font=>$font,
											-textvariable=>\$new_name,
											-validate=>'key',
											-vcmd=>sub{ 
														my $flag = 1;
														foreach (@displayed_list){$flag = 0 if $_[0] eq $_}
														if ($_[0] ne '' && $flag){
															$prompt_box->Subwidget('B_Done')->configure(-state=>'active');
														}
														else {
															$prompt_box->Subwidget('B_Done')->configure(-state=>'disabled');															
														}
													return 1;	
													}
											)->pack(-side=>'top');
		
		$prompt_box->Subwidget('B_Done')->configure(-font=>['Helvetica',11,'bold'],-state=>'disabled');											
		$prompt_box->Subwidget('B_Edit File')->configure(-font=>['Helvetica',11,'bold']);
		$prompt_box->Subwidget('B_Cancel')->configure(-font=>['Helvetica',11,'bold']);
		$prompt_box->resizable(0,0);
		$prompt_box->configure(-popover=>$Configuration_window);   	
		my $button = $prompt_box->Show;


		if ($button eq 'Edit File'){
			
			foreach my $i (0..@buttons-1){
				$buttons[$i] = $new_name if $old_name eq $buttons[$i];	
			}
			
			
			my $file = $main->getOpenFile(-filetypes=>[['Wave Files','.wav'],['All Files','.*',],],-title=>'Choose a sound file');

			
			if (defined $file){
				
				${file{$new_name}} = $file;
				$Button_list->configure(-listvariable=>\@buttons);
				@displayed_list = @buttons;
			}
		
		}
		
		if ($button eq 'Done'){
			
			foreach my $i (0..@buttons-1){
				$buttons[$i] = $new_name if ($old_name eq $buttons[$i]);	
			}
			
			${file{$new_name}} = ${file{$old_name}};
			$Button_list->configure(-listvariable=>\@buttons);
			@displayed_list = @buttons;
		}

		
	}#end sub Edit
	
	sub Change_num_of_buttons {  
		$default_flag = 1 if $button_changing > $rows*$columns; 
    	${frame{$_}}->destroy foreach (keys %frame);
		delete @button{keys %button};
		delete @frame{keys %frame};
		$number_of_buttons = $button_changing;
		&Create_Buttons;
	}#end Num_of_buttons	

}#end sub Configuration_Manager

###########################################################################################

sub main_menu {
	
my $menubar = $main->Menu();
my $sizes;
my $volume_choice;
my $volume_displayed_choice;

$main->configure(-menu => $menubar,-borderwidth=>0);

$menubar->Cascade(qw/-label ~File -tearoff 0 -menuitems/ =>
    [
	 [Button => 'Exit',        -command =>sub{&Save_Settings; exit;}],
	],
    ); 

my $mb2 = $menubar->Cascade(qw/-label ~Settings -tearoff 0 -menuitems/ =>
    [
    [Button => 'Configure Buttons',        -command =>sub{&Configuration_Manager}],
    [Button => 'Layout',        -command =>sub{&Change_Layout}],
    ]);  
    
    $sizes = $mb2->cascade(-label => 'Button Size', -tearoff => 0);
    $sizes->radiobutton(-label => 'Small', -variable => \$font_size, -value=>8,-command=>sub{&Change_size_of_buttons($font_size)});
    $sizes->radiobutton(-label => 'Medium', -variable => \$font_size, -value=>12,-command=>sub{&Change_size_of_buttons($font_size)});
    $sizes->radiobutton(-label => 'Large', -variable => \$font_size, -value=>16,-command=>sub{&Change_size_of_buttons($font_size)});   

    $volume_choice = $mb2->cascade(-label => 'Volume Orientation', -tearoff => 0);
    $volume_choice->radiobutton(-label => 'Horizontal', -variable => \$orient_vol, -value=>'horizontal',-command=>sub{&Change_Volume});
    $volume_choice->radiobutton(-label => 'Vertical', -variable => \$orient_vol, -value=>'vertical',-command=>sub{&Change_Volume});

    $volume_displayed_choice = $mb2->cascade(-label => 'Volume Visible', -tearoff => 0);
    $volume_displayed_choice->radiobutton(-label => 'Yes', -variable => \$volume_visible, -value=>1,-command=>sub{&Volume_Visible});
    $volume_displayed_choice->radiobutton(-label => 'No', -variable => \$volume_visible, -value=>0,-command=>sub{&Volume_Visible});

    
        
$menubar->Cascade(qw/-label ~Help -tearoff 0 -menuitems/ =>
     [
	 [Button => 'Contents',   -command =>sub{}],
	 [Separator => ''],
	 [Button => 'About',        -command =>sub{}],
	],
    );
    
sub Change_size_of_buttons {  
		${frame{$_}}->destroy foreach (keys %frame);
		delete @button{keys %button};
		delete @frame{keys %frame};
		$button_font = ['Helvetica',$_[0],'bold'];
		&Create_Buttons;
		&Save_Settings;
	}#end Num_of_buttons
	
sub Change_Layout{
		
our $prompt_box = $main->Dialog(
			 -title          =>'Change Layout' ,
			 -buttons=>['OK','Cancel','Default'],
		);

my $entry_frame = $prompt_box->Frame(-bg=>'red')->pack(-side=>'top',-fill=>'x',-expand=>1,-padx=>41);		
				
my $row_frame = $entry_frame->Frame()->pack(-side=>'left');
	$row_frame->Label(-text=>"Rows:",-font=>['Helvetica',13,'bold'],-justify=>'center')->pack(-side=>'top');
	$row_frame->Entry(-width=>8,-justify=>'center',-font=>$font,-textvariable=>\$rows,-validate=>'key',-vcmd=>sub{&validate_row})->pack(-side=>'top',-padx=>8);

my $col_frame = $entry_frame->Frame()->pack(-side=>'right');
	$col_frame->Label(-text=>"Columns:",-font=>['Helvetica',13,'bold'],-justify=>'center')->pack(-side=>'top');
	$col_frame->Entry(-width=>8,-justify=>'center',-font=>$font,-textvariable=>\$columns,-validate=>'key',-vcmd=>sub{&validate_col})->pack(-side=>'top',-padx=>8);
			
		$prompt_box->Subwidget('B_OK')->configure(-font=>['Helvetica',11,'bold'],-state=>'disabled');											
		$prompt_box->Subwidget('B_Cancel')->configure(-font=>['Helvetica',11,'bold']);
		$prompt_box->Subwidget('B_Default')->configure(-font=>['Helvetica',11,'bold']);
		$prompt_box->resizable(0,0);
		$prompt_box->configure(-popover=>$main);   	
		my $button = $prompt_box->Show;
		
		if ($button eq 'OK' || $button eq 'Default'){
			$default_flag = ($button eq 'OK'?0:1);
			$number_of_buttons = @buttons if $button eq 'Default';
			$number_of_buttons = $rows*$columns if $button eq 'OK';
			${frame{$_}}->destroy foreach (keys %frame);
			delete @button{keys %button};
			delete @frame{keys %frame};
			&Create_Buttons;
			&Save_Settings;	
		}
		
sub validate_row {

if ($_[0] =~ /^[0-9]*$/ && $_[0] ne '' && $_[0] > 0){
	if (($_[0]*$columns) <= 64){
		$prompt_box->Subwidget('B_OK')->configure(-state=>'active');												
	}
	else {$prompt_box->Subwidget('B_OK')->configure(-state=>'disabled');}
}
else {$prompt_box->Subwidget('B_OK')->configure(-state=>'disabled');}	
return 1;		
}#end validate_row

sub validate_col {

if ($_[0] =~ /^[0-9]*$/ && $_[0] ne '' && $_[0] > 0){
	if (($rows*$_[0]) <= 64){
		$prompt_box->Subwidget('B_OK')->configure(-state=>'active');											
	}	
	else {$prompt_box->Subwidget('B_OK')->configure(-state=>'disabled');}
}
else {$prompt_box->Subwidget('B_OK')->configure(-state=>'disabled');}
return 1;	
}#end validate_col
	
}#end change_layout
	
}#end main_menu
